<?php
require_once(__DIR__ . '/../Helper/sdk/payments.php');
class EService_Payment_Model_Payment extends Mage_Payment_Model_Method_Abstract {

    protected $_code = 'eservice_payment';
//     protected $_formBlockType = 'eservice_payment/form';
    protected $_isGateway = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = false;
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;
    protected $_canVoid = true;
    protected $_canUseInternal = true;
    protected $_canUseCheckout = true;
    protected $_canUseForMultishipping = false;
    protected $_canSaveCc = false;    
    protected $_sandbox = false;
    protected $_isInitializeNeeded      = true;
    
    
    protected $_logenabled = false;
    protected $_logfile = 'eservice_payment.txt';
    
    
    public function __construct() {
        parent::__construct(); 
        if($this->getConfigData('payment_environment') == 1){
            $this->_sandbox = true; 
        }
        // init the SDK configuration settings
        EServicePayments\Config::$MerchantId = $this->getConfigData('merchant_id');
        EServicePayments\Config::$Password = $this->getConfigData('merchant_password');
        if ($this->_sandbox){
            EServicePayments\Config::$TestUrls["SessionTokenRequestUrl"] = $this->getConfigData('token_sandbox_url');
            EServicePayments\Config::$TestUrls["PaymentOperationActionUrl"] = $this->getConfigData('payments_sandbox_url');
            EServicePayments\Config::$TestUrls["JavaScriptUrl"] = $this->getConfigData('js_sandbox_url');
            EServicePayments\Config::$TestUrls["BaseUrl"] =  $this->getConfigData('cashier_sandbox_url');
            EServicePayments\Config::test();
        }else{
            EServicePayments\Config::$ProductionUrls["SessionTokenRequestUrl"] = $this->getConfigData('token_production_url');
            EServicePayments\Config::$ProductionUrls["PaymentOperationActionUrl"] = $this->getConfigData('payments_production_url');
            EServicePayments\Config::$ProductionUrls["JavaScriptUrl"] = $this->getConfigData('js_production_url');
            EServicePayments\Config::$ProductionUrls["BaseUrl"] = $this->getConfigData('cashier_production_url');
            EServicePayments\Config::production();
        }
    }
    public function test(){
        return $this->getConfigData('order_status');
    }
    /**
     * Instantiate state and set it to state object
     *
     * @param string $paymentAction
     * @param Varien_Object
     */
    public function initialize($paymentAction, $stateObject)
    {
        switch ($paymentAction) {
            case self::ACTION_AUTHORIZE:
            case self::ACTION_AUTHORIZE_CAPTURE:
                $payment = $this->getInfoInstance();
                $order = $payment->getOrder();
                
                $order->setState(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, true, '', false);
                
                
                $stateObject->setState(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT);
                $stateObject->setStatus('pending');
                $stateObject->setIsNotified(false);
                break;
            default:
                break;
        }
    }
    /**
     * @return Mage_Checkout_Model_Session
     */
    public function getCheckout() {
        return Mage::getSingleton('checkout/session');
    }
    /**
    * Return Order place redirect url
    *
    * @return string
    */
    public function getOrderPlaceRedirectUrl() {
        //when you click on place order you will be redirected on this url depending on the payment mode
        if($this->getConfigData('payment_mode') == 'iframe'){
            return Mage::getUrl('eservice/payment/iframe');
        }else{
            return Mage::getUrl('eservice/payment/redirect');
        }
    }
    public function validate()
    {
        parent::validate();
        
       
        return $this;
    }
   

    public function getGatewayJSUrl() {
        if($this->_sandbox) {
            return $this->getConfigData('js_sandbox_url');
        }
        return $this->getConfigData('js_production_url');
    }
    public function getGatewayTokenUrl() {
        if($this->_sandbox) {
            return $this->getConfigData('token_sandbox_url');
        }
        return $this->getConfigData('token_production_url');
    }
    public function getGatewayCashierUrl() {
        if($this->_sandbox) {
            return $this->getConfigData('cashier_sandbox_url');
        }
        return $this->getConfigData('cashier_production_url');
    }
    public function getGatewayPaymentUrl() {
        if($this->_sandbox) {
            return $this->getConfigData('payments_sandbox_url');
        }
        return $this->getConfigData('payments_production_url');
    }
    /**
     * 	Return the allowOriginUrl
     * 	@return string
     */
    public function getAllowOriginUrl() {
        // to get only the site's domain url name to assign to the parameter allowOriginUrl .
        //otherwise it will encounter a CORS issue when wordpress deployed inside a subfolder of the web server.
        $parse_result = parse_url(Mage::getUrl());
        if(isset($parse_result['port'])){
            $allowOriginUrl = $parse_result['scheme']."://".$parse_result['host'].":".$parse_result['port'];
        }else{
            $allowOriginUrl = $parse_result['scheme']."://".$parse_result['host'];
        }
        return $allowOriginUrl;
    }
    /**
     * 	Return Array parameters to call for the Auth/Purchase token
     * 	@return mixed
     */
    public function getGatewayTokenArgs(){
        $order_id = $this->getCheckout()->getLastRealOrderId();
        $orderModel = Mage::getModel('sales/order');
        $order = $orderModel->loadByIncrementId($order_id);
        $b = $order->getBillingAddress();
        $billingAdress=$b->getData();
        $orderData = $order->getData();
        $amount = number_format($order->getGrandTotal(), 2, '.', '');
        
        $paymentSolutionId = '';
        
        if($this->getConfigData('payment_action') == self::ACTION_AUTHORIZE){
            $action = 'AUTH';
        }else{
            $action = 'PURCHASE';
        }
        
        $allowOriginUrl = $this->getAllowOriginUrl();
        $token_parameters=  array(
            'merchantId' => $this->getConfigData('merchant_id'),
            'password' => $this->getConfigData('merchant_password'),
            'brandId' => $this->getConfigData('brand_id'),
            'action' => $action,
            'amount' => $amount,
            'currency' => $order->getBaseCurrencyCode(),
            'country' => Mage::getStoreConfig('general/country/default'),
            'language' => substr(Mage::app()->getLocale()->getLocaleCode(), 0,2), //to get the first 2 letters of the current language code
            'paymentSolutionId' => $paymentSolutionId,
            'timestamp' => round(microtime(true) * 1000),
            'channel' => 'ECOM',
            'allowOriginUrl' => $allowOriginUrl,
            "merchantNotificationUrl" =>$this->getNotifyurl($order_id),
            "merchantLandingPageUrl" => $this->getLandingPageurl($order_id),
            "customerBillingAddressPostalCode" => $billingAdress['postcode'],
            "customerBillingAddressCity" => $billingAdress['city'],
            "customerBillingAddressCountry" => $b->getCountryModel()->getcountryId(),
            "customerBillingAddressStreet" => $b->getStreet(1),
            "customerFirstName" => $orderData['customer_firstname'],
            "customerLastName" => $orderData['customer_lastname'],
        );
        $this->logData(json_encode($token_parameters));
        return $token_parameters;
    }
    /**
     * 	call the API Gateway
     */
    public function generateGatewayRedirectRequest(){
        if ( $this->getConfigData('merchant_id') === null || $this->getConfigData('merchant_id') === ''
            || $this->getConfigData('merchant_password') === null || $this->getConfigData('merchant_password') === '' ) {
                return false;
            }
            $post_data = $this->getGatewayTokenArgs();
            try {
                $payments = new EServicePayments\Payments();
                if($this->getConfigData('payment_action') == self::ACTION_AUTHORIZE){
                    $payments_request = $payments->auth();
                }else{
                    $payments_request = $payments->purchase();
                }
                $payments_request->
                brandId($post_data['brandId'])->
                action($post_data['action'])->
                allowOriginUrl($post_data['allowOriginUrl'])->
                merchantLandingPageUrl($post_data['merchantLandingPageUrl'])->
                merchantNotificationUrl($post_data['merchantNotificationUrl'])->
                timestamp($post_data['timestamp'])->
                channel($post_data['channel'])->
                language($post_data['language'])->
                amount($post_data['amount'])->
                paymentSolutionId($post_data['paymentSolutionId'])->
                currency($post_data['currency'])->
                country($post_data['country'])->
                customerFirstName($post_data['customerFirstName'])->
                customerLastName($post_data['customerLastName'])->
                customerBillingAddressPostalCode($post_data['customerBillingAddressPostalCode'])->
                customerBillingAddressCity($post_data['customerBillingAddressCity'])->
                customerBillingAddressCountry($post_data['customerBillingAddressCountry'])->
                customerBillingAddressStreet($post_data['customerBillingAddressStreet']);
                $result = $payments_request->token();
                $this->logData('call the API Gateway:'.json_encode($result->result));
            } catch (Exception $e) {
                $this->logData(json_encode($e->getMessage()));
                return false;
            }
            if($result->result != 'success'){
                return false;
            }
            $response_data = array();
            $response_data['token'] = $result['token'];
            $response_data['merchantId'] = $this->getConfigData('merchant_id');
            $response_data['integrationMode'] = $this->getConfigData('payment_mode');
            return $response_data;
    }
    
 
    /**
     * 	Return notification url on which gateway send notification
     * 	@return mixed
     */
    public function getNotifyurl($orderid) {
        return Mage::getUrl('eservice/payment/callback', array("orderid" => $orderid));
    }
    
    /**
     * 	Return gateway landing page url
     * 	@return mixed
     */
    public function getLandingPageurl($orderid) {
        return Mage::getUrl('eservice/payment/response', array("orderid" => $orderid));
    }
    public function logData($message) {
        if($this->_logenabled) {
            Mage::log($message, null, $this->_logfile, true);
        }
    }
    /**
     * 	Return the Session Token for Status Check 
     * 	@return mixed
     */
    public function getCheckStatusTokenArgs($merchantTxId){
        return array(
            'merchantId' => $this->getConfigData('merchant_id'),
            'password' => $this->getConfigData('merchant_password'),
            "allowOriginUrl" => $this->getAllowOriginUrl(),
            "action" => "GET_STATUS",
            "timestamp" => round(microtime(true) * 1000),
            "merchantTxId" => $merchantTxId
        );
    }
    /**
     * 	Call the Gateway for Status Check
     * 	@return mixed
     */
    public function generateCheckStatusRequest($order_id,$merchantTxId){
        if ( $this->getConfigData('merchant_id') === null || $this->getConfigData('merchant_id') === ''
            || $this->getConfigData('merchant_password') === null || $this->getConfigData('merchant_password') === '' ) {
                return false;
            }
            $this->logData('Check Status for order:'.$order_id);
            $data = $this->getCheckStatusTokenArgs($merchantTxId);
            try {
                $payments = new EServicePayments\Payments();
                $status_check = $payments->status_check();
                $status_check->merchantTxId($data['merchantTxId'])->
                action('GET_STATUS')->
                timestamp($data['timestamp'])->
                allowOriginUrl($data['allowOriginUrl']);
                $result = $status_check->execute();
                $this->logData('result of Check Status:'.json_encode($result->result));
            } catch (Exception $e) {
                return false;
            }
            if($result->result !== 'success' ){
                return false;
            }else{
                $order = Mage::getModel('sales/order')->loadByIncrementId($order_id);
                $payment = $order->getPayment();
                if($payment->getAdditionalInformation('gateway_merchantTxId')){
                    //if the payment has a gateway_merchantTxId already then it means the order payment has been notified, do not need to redo it again
                    return true;
                }
                
                if($result->status == 'SET_FOR_CAPTURE' || $result->status == 'CAPTURED'){
                    //PURCHASE was successful
                    $payment->setAdditionalInformation('gateway_merchantTxId',$merchantTxId);
                    $payment->setTransactionId($merchantTxId);
                    $payment->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE);
                    $payment->save();
                    $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true,'Payment paid')->save();
                    $this->generateInvoice($order);
                }else if($result->status == 'NOT_SET_FOR_CAPTURE'){
                    // AUTH was successful
                    $payment->setAdditionalInformation('gateway_merchantTxId',$merchantTxId);
                    $payment->setTransactionId($merchantTxId);
                    $transaction = $payment->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_AUTH);
                    $transaction->setIsClosed(0);
                    $transaction->save();
                    
                    $payment->save();
                    $order->setState(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, true, 'authorized', false)->save();
                }else{
                    //the order payment was declined or canceled.
                    $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true,'Payment error')->save();
                    return false;
                }
                return true;
            }
    }
    /**
     * if the payment mode is SALE, then it will call this method to capture the payment
     *
     */
    public function generateInvoice($order){
        try {
            if (!$order->getId()) {
                Mage::throwException(Mage::helper('payment')->__('The order no longer exists.'));
            }
            if(!$order->canInvoice()) {
                Mage::throwException(Mage::helper('payment')->__('The order does not allow an invoice to be created.'));
            }
            $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
            $invoice->register();
            $invoice->getOrder()->setIsInProcess(true);
            $transactionSave = Mage::getModel('core/resource_transaction')
            ->addObject($invoice)
            ->addObject($invoice->getOrder());
            $transactionSave->save();
            return $invoice;
        } catch (Exception $e) {
            $this->logData('generateInvoice: '.json_encode($e->getMessage()));
        }
    }
    public function authorize(Varien_Object $payment, $amount){
        parent::authorize($payment, $amount);
        $this->logData('authorize: '.$amount);
        return $this;
    }
    /**
     * Send capture request to gateway
     *
     * @param Varien_Object $payment
     * @param decimal $amount
     * @return Mage_Authorizenet_Model_Directpost
     * @throws Mage_Core_Exception
     */
    public function capture(Varien_Object $payment, $amount){
        if ($amount <= 0) {
            Mage::throwException(Mage::helper('payment')->__('Invalid amount for capture.'));
        }
//         $merchantTxId = $payment->getLastTransId();
        $merchantTxId = $payment->getAdditionalInformation('gateway_merchantTxId');
        $data = array(
            'merchantId' => $this->getConfigData('merchant_id'),
            'password' => $this->getConfigData('merchant_password'),
            "allowOriginUrl" => $this->getAllowOriginUrl(),
            "action" => "CAPTURE",
            "timestamp" => round(microtime(true) * 1000),
            "originalMerchantTxId" => $merchantTxId,
            "amount" => $amount
        );
        try {
            $payments = new EServicePayments\Payments();
            $capture = $payments->capture();
            $capture->originalMerchantTxId($data['originalMerchantTxId'])->
            action('CAPTURE')->
            amount($data['amount'])->
            timestamp($data['timestamp'])->
            allowOriginUrl($data['allowOriginUrl']);
            $result = $capture->execute();
        } catch (Exception $e) {
            Mage::throwException('Capture error: '.$e->getMessage());
            return;
        }
        if($result->result == 'success' && $result->status == 'SET_FOR_CAPTURE' ) {
            $order = $payment->getOrder();
            $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true,'Payment Captured')->save();
        }else{
            if (!is_array($result->errors) && strpos($result->errors, 'current status: SUCCESS') !== false) {
                //the order has been capture already in the Gateway
            }else{
                Mage::throwException(json_encode($result->errors));
            }
        }
        return $this;
        
    }
    public function cancel(Varien_Object $payment){
        $this->logData('cancel: ');
        return $this->void($payment);
    }
    public function void(Varien_Object $payment){
        $merchantTxId = $payment->getAdditionalInformation('gateway_merchantTxId');
        $data = array(
            'merchantId' => $this->getConfigData('merchant_id'),
            'password' => $this->getConfigData('merchant_password'),
            "allowOriginUrl" => $this->getAllowOriginUrl(),
            "action" => "VOID",
            "timestamp" => round(microtime(true) * 1000),
            "originalMerchantTxId" => $merchantTxId
        );
        $this->logData('VOID data parameters: '.json_encode($data));
        try {
            $payments = new EServicePayments\Payments();
            $void = $payments->void();
            $void->originalMerchantTxId($data['originalMerchantTxId'])->
            action('VOID')->
            timestamp($data['timestamp'])->
            allowOriginUrl($data['allowOriginUrl']);
            $result = $void->execute();
        } catch (Exception $e) {
            Mage::throwException('Void error: '.$e->getMessage());
            return $this;
        }
        
        if($result->result == 'success' && $result->status == 'VOID' ) {
            return $this;
        } else {
            $order = $payment->getOrder();
            $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true,'Payment voided error')->save();
            Mage::throwException(Mage::helper('payment')->__('Payment void error'));
        }
        //         return $this;
    }
    /**
     * Send refund request to gateway
     *
     * @param Varien_Object $payment
     * @param decimal $amount
     * @return Mage_Authorizenet_Model_Directpost
     * @throws Mage_Core_Exception
     */
    public function refund(Varien_Object $payment, $amount){
        parent::refund($payment, $amount);
        if ($amount <= 0) {
            Mage::throwException(Mage::helper('payment')->__('Invalid amont for refund.'));
        }
//         $merchantTxId = $payment->getLastTransId();
        $merchantTxId = $payment->getAdditionalInformation('gateway_merchantTxId');
        $data = array(
            'merchantId' => $this->getConfigData('merchant_id'),
            'password' => $this->getConfigData('merchant_password'),
            "allowOriginUrl" => $this->getAllowOriginUrl(),
            "action" => "REFUND",
            "amount" => $amount,
            "timestamp" => round(microtime(true) * 1000),
            "originalMerchantTxId" => $merchantTxId
        );
        $this->logData('refund data parameters: '.json_encode($data));
        try {
            $payments = new EServicePayments\Payments();
            $refund = $payments->refund();
            $refund->originalMerchantTxId($data['originalMerchantTxId'])->
            action('REFUND')->
            amount($data['amount'])->
            timestamp($data['timestamp'])->
            allowOriginUrl($data['allowOriginUrl']);
            $result = $refund->execute();
        } catch (Exception $e) {
            Mage::throwException('Refund error: '.$e->getMessage());
            return $this;
        }
        if($result->result == 'success' && $result->status == 'SET_FOR_REFUND' ) {
            return $this;
        }else if(!is_array($result->errors) && strpos($result->errors, 'Transaction not refundable: Original transaction not SUCCESS') !== false){
            //if the order was authorized + captured, the status in the Gateway system is still showing NOT_SET_FOR_CAPTURE, the refund can not be excuted
            Mage::throwException(Mage::helper('payment')->__('The order is in capture process queue, can not refund now!'));
        }else{
            Mage::throwException(json_encode($result->errors));
        }
        return $this;
    }
    /**
     * curl request post
     * @param string $url
     * @param array $post_data
     */
    public function request_post($url = '', $post_data = array()) {
        if (empty($url) || empty($post_data)) {
            return false;
        }
        
        $o = "";
        foreach ( $post_data as $k => $v )
        {
            $o.= "$k=" . urlencode( $v ). "&" ;
        }
        $post_data = substr($o,0,-1);
        
        $postUrl = $url;
        $curlPost = $post_data;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$postUrl);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $curlPost);
        $data = curl_exec($ch);
        curl_close($ch);
        
        return $data;
    }
    
}

?>
